@@ -30,8 +30,8 @@ GST_DEBUG_CATEGORY (libvisual_debug);
 static void
 libvisual_log_handler (const char *message, const char *funcname, void *priv)
 {
-  GST_CAT_LEVEL_LOG (libvisual_debug, (GstDebugLevel) (priv), NULL, "%s - %s",
-      funcname, message);
+  GST_CAT_LEVEL_LOG (libvisual_debug, (GstDebugLevel) GPOINTER_TO_INT (priv),
+      NULL, "%s - %s", funcname, message);
 }
 
 /*
@@ -58,21 +58,10 @@ gst_visual_actor_plugin_is_gl (VisObject * plugin, const gchar * name)
   gboolean is_gl;
   gint depth;
 
-#if !defined(VISUAL_API_VERSION)
-
-  depth = VISUAL_PLUGIN_ACTOR (plugin)->depth;
-  is_gl = (depth == VISUAL_VIDEO_DEPTH_GL);
-
-#elif VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
-
   depth = VISUAL_ACTOR_PLUGIN (plugin)->vidoptions.depth;
   /* FIXME: how to figure this out correctly in 0.4? */
   is_gl = (depth & VISUAL_VIDEO_DEPTH_GL) == VISUAL_VIDEO_DEPTH_GL;
 
-#else
-# error what libvisual version is this?
-#endif
-
   if (!is_gl) {
     GST_DEBUG ("plugin %s is not a GL plugin (%d), registering", name, depth);
   } else {
@@ -97,13 +86,14 @@ plugin_init (GstPlugin * plugin)
 #endif
 
   visual_log_set_verboseness (VISUAL_LOG_VERBOSENESS_LOW);
-  visual_log_set_info_handler (libvisual_log_handler, (void *) GST_LEVEL_INFO);
+  visual_log_set_info_handler (libvisual_log_handler,
+      GINT_TO_POINTER (GST_LEVEL_INFO));
   visual_log_set_warning_handler (libvisual_log_handler,
-      (void *) GST_LEVEL_WARNING);
+      GINT_TO_POINTER (GST_LEVEL_WARNING));
   visual_log_set_critical_handler (libvisual_log_handler,
-      (void *) GST_LEVEL_ERROR);
+      GINT_TO_POINTER (GST_LEVEL_ERROR));
   visual_log_set_error_handler (libvisual_log_handler,
-      (void *) GST_LEVEL_ERROR);
+      GINT_TO_POINTER (GST_LEVEL_ERROR));
 
   if (!visual_is_initialized ())
     if (visual_init (NULL, NULL) != 0)
@@ -111,11 +101,7 @@ plugin_init (GstPlugin * plugin)
 
   list = visual_actor_get_list ();
 
-#if !defined(VISUAL_API_VERSION)
-  count = visual_list_count (list);
-#elif VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
   count = visual_collection_size (VISUAL_COLLECTION (list));
-#endif
 
   for (i = 0; i < count; i++) {
     VisPluginRef *ref = visual_list_get (list, i);
