@@ -39,63 +39,6 @@
 #define NSWindowStyleMaskMiniaturizable      NSMiniaturizableWindowMask
 #endif
 
-static const gchar*
-gst_gl_cocoa_keycode_to_keyname (NSEvent *event)
-{
-  switch ([event keyCode]) {
-    /* The key codes are taken from Events.h which is included only from
-     * Carbon/Carbon.h, which is deprecated in macOS 10.8, using their values
-     * directly as cocoa does not provide any replacement for those.
-     */
-    case 0x18: return "equal";
-    case 0x1B: return "minus";
-    case 0x1E: return "bracketright";
-    case 0x21: return "bracketleft";
-    case 0x27: return "apostrophe";
-    case 0x29: return "semicolon";
-    case 0x2A: return "backslash";
-    case 0x2B: return "comma";
-    case 0x2C: return "slash";
-    case 0x2F: return "period";
-    case 0x32: return "grave";
-
-    case 0x24: return "Return";
-    case 0x30: return "Tab";
-    case 0x31: return "space";
-    case 0x33: return "BackSpace";
-    case 0x35: return "Escape";
-    case 0x37: return "Command";
-    case 0x38: return "Shift_L";
-    case 0x39: return "Caps_Lock";
-    case 0x3A: return "Alt_L";
-    case 0x3B: return "Control_L";
-    case 0x3C: return "Shift_R";
-    case 0x3D: return "Alt_R";
-    case 0x3E: return "Control_R";
-    case 0x7A: return "F1";
-    case 0x78: return "F2";
-    case 0x63: return "F3";
-    case 0x76: return "F4";
-    case 0x60: return "F5";
-    case 0x61: return "F6";
-    case 0x62: return "F7";
-    case 0x64: return "F8";
-    case 0x65: return "F9";
-    case 0x6D: return "F10";
-    case 0x67: return "F11";
-    case 0x6F: return "F12";
-    case 0x73: return "Home";
-    case 0x77: return "End";
-    case 0x74: return "Page_Up";
-    case 0x79: return "Page_Down";
-    case 0x7B: return "Left";
-    case 0x7C: return "Right";
-    case 0x7D: return "Down";
-    case 0x7E: return "Up";
-    default: return [event characters].UTF8String;
-  }
-}
-
 /* =============================================================*/
 /*                                                              */
 /*               GstGLNSWindow declaration                      */
@@ -111,7 +54,6 @@ gst_gl_cocoa_keycode_to_keyname (NSEvent *event)
     backing: (NSBackingStoreType) bufferingType
     defer: (BOOL) flag screen: (NSScreen *) aScreen
     gstWin: (GstGLWindowCocoa *) window;
-- (void) resize: (int) width height: (int) height;
 - (void) setClosed;
 - (BOOL) isClosed;
 - (BOOL) canBecomeMainWindow;
@@ -262,13 +204,15 @@ gst_gl_window_cocoa_create_window (GstGLWindowCocoa *window_cocoa)
   NSRect rect = NSMakeRect (0, y, priv->preferred_width, priv->preferred_height);
   NSRect windowRect = NSMakeRect (0, y, priv->preferred_width, priv->preferred_height);
   GstGLContext *context = gst_gl_window_get_context (window);
+  GstGLContextCocoa *context_cocoa;
   GstGLCAOpenGLLayer *layer;
   GstGLNSView *glView;
 
   if (!context)
     return FALSE;
 
-  layer = [[GstGLCAOpenGLLayer alloc] initWithGstGLWindow:window];
+  context_cocoa = GST_GL_CONTEXT_COCOA (context);
+  layer = [[GstGLCAOpenGLLayer alloc] initWithGstGLContext:context];
   layer.autoresizingMask = kCALayerWidthSizable | kCALayerHeightSizable;
   layer.needsDisplayOnBoundsChange = YES;
   glView = [[GstGLNSView alloc] initWithFrameLayer:window_cocoa rect:windowRect layer:layer];
@@ -294,14 +238,16 @@ gst_gl_window_cocoa_create_window (GstGLWindowCocoa *window_cocoa)
     gst_gl_window_cocoa_set_window_handle (window,
         (guintptr) priv->external_view);
 
-  [NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];
-
   return TRUE;
 }
 
 static gboolean
 gst_gl_window_cocoa_open (GstGLWindow *window, GError **err)
 {
+  GstGLWindowCocoa *window_cocoa;
+
+  window_cocoa = GST_GL_WINDOW_COCOA (window);
+
   return TRUE;
 }
 
@@ -314,13 +260,6 @@ _close_window (gpointer * data)
 
   [view removeFromSuperview];
 
-  if (!window_cocoa->priv->external_view) {
-    GstGLNSWindow *internal_win_id =
-        (__bridge GstGLNSWindow *) window_cocoa->priv->internal_win_id;
-
-    [internal_win_id close];
-  }
-
   CFBridgingRelease (window_cocoa->priv->internal_win_id);
   CFBridgingRelease (window_cocoa->priv->internal_view);
   window_cocoa->priv->internal_win_id = NULL;
@@ -330,8 +269,8 @@ _close_window (gpointer * data)
 static void
 gst_gl_window_cocoa_close (GstGLWindow * window)
 {
-  _gst_gl_invoke_on_main ((GstGLWindowCB) _close_window,
-      gst_object_ref (window), (GDestroyNotify) gst_object_unref);
+  _invoke_on_main ((GstGLWindowCB) _close_window, gst_object_ref (window),
+      (GDestroyNotify) gst_object_unref);
 }
 
 static guintptr
@@ -350,7 +289,6 @@ gst_gl_window_cocoa_set_window_handle (GstGLWindow * window, guintptr handle)
   priv = window_cocoa->priv;
 
   if (priv->internal_win_id) {
-    GstGLWindowCocoa *window_cocoa2 = gst_object_ref (window_cocoa);
     if (handle) {
       priv->external_view = (gpointer)handle;
       priv->visible = TRUE;
@@ -360,6 +298,7 @@ gst_gl_window_cocoa_set_window_handle (GstGLWindow * window, guintptr handle)
       priv->visible = FALSE;
     }
 
+
     dispatch_async (dispatch_get_main_queue (), ^{
       GstGLNSWindow *internal_win_id =
           (__bridge GstGLNSWindow *)window_cocoa->priv->internal_win_id;
@@ -374,7 +313,6 @@ gst_gl_window_cocoa_set_window_handle (GstGLWindow * window, guintptr handle)
       [external_view setAutoresizesSubviews: YES];
       [view setFrame: [external_view bounds]];
       [view setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
-      gst_object_unref (window_cocoa2);
     });
   } else {
     /* no internal window yet so delay it to the next drawing */
@@ -391,10 +329,6 @@ _show_window (gpointer data)
   GstGLNSWindow *internal_win_id = (__bridge GstGLNSWindow *)priv->internal_win_id;
 
   GST_DEBUG_OBJECT (window_cocoa, "make the window available\n");
-
-  /* Preferred size might not yet be set when create_window is called,
-   * so we need to resize the window here to be sure. */
-  [internal_win_id resize: priv->preferred_width height: priv->preferred_height];
   [internal_win_id makeMainWindow];
   [internal_win_id orderFrontRegardless];
   [internal_win_id setViewsNeedDisplay:YES];
@@ -418,8 +352,8 @@ gst_gl_window_cocoa_show (GstGLWindow * window)
     }
 
     if (!priv->external_view && !priv->visible)
-      _gst_gl_invoke_on_main ((GstGLWindowCB) _show_window,
-          gst_object_ref (window), (GDestroyNotify) gst_object_unref);
+      _invoke_on_main ((GstGLWindowCB) _show_window, gst_object_ref (window),
+          (GDestroyNotify) gst_object_unref);
   }
 }
 
@@ -651,8 +585,8 @@ gst_gl_window_cocoa_set_render_rectangle (GstGLWindow * window, gint x, gint y,
  render->rect.w = width;
  render->rect.h = height;
 
- _gst_gl_invoke_on_main ((GstGLWindowCB) _set_render_rectangle, render,
-     (GDestroyNotify) _free_set_render_rectangle);
+ _invoke_on_main ((GstGLWindowCB) _set_render_rectangle, render,
+                  (GDestroyNotify) _free_set_render_rectangle);
 
  return TRUE;
 }
@@ -700,15 +634,6 @@ gst_gl_window_cocoa_controls_viewport (GstGLWindow * window)
   return self;
 }
 
-- (void) resize: (gint) width height: (gint) height {
-  NSRect frame = [super frame];
-  if (frame.size.width == width && frame.size.height == height)
-    return;
-
-  frame.size = NSMakeSize (width, height);
-  [super setFrame:frame display:YES];
-}
-
 - (void) setClosed {
   m_isClosed = YES;
 }
@@ -785,131 +710,9 @@ close_window_cb (gpointer data)
 
   [self setWantsBestResolutionOpenGLSurface:YES];
 
-  /* Set up mouse tracking */
-  NSTrackingArea *trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds]
-      options:(NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveInKeyWindow)
-      owner:self userInfo:nil];
-  [self addTrackingArea:trackingArea];
-
   return self;
 }
 
-- (BOOL)acceptsFirstResponder {
-  return YES;
-}
-
-- (void)updateTrackingAreas {
-  [super updateTrackingAreas];
-
-  for (NSTrackingArea *area in [self trackingAreas]) {
-    [self removeTrackingArea:area];
-  }
-
-  NSTrackingArea *trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds]
-      options:(NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveInKeyWindow)
-      owner:self userInfo:nil];
-  [self addTrackingArea:trackingArea];
-}
-
-- (void)keyDown:(NSEvent *)event {
-  const gchar *event_name = "key-press";
-  const gchar *key_str = gst_gl_cocoa_keycode_to_keyname(event);
-
-  if (key_str && *key_str) {
-    gst_gl_window_send_key_event (GST_GL_WINDOW (window_cocoa), event_name, key_str);
-  }
-
-  [super keyDown:event];
-}
-
-- (void)keyUp:(NSEvent *)event {
-  const gchar *event_name = "key-release";
-  const gchar *key_str = gst_gl_cocoa_keycode_to_keyname(event);
-
-  if (key_str && *key_str) {
-    gst_gl_window_send_key_event (GST_GL_WINDOW (window_cocoa), event_name, key_str);
-  }
-
-  [super keyUp:event];
-}
-
-- (void)mouseDown:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-press", 1, location.x, location.y);
-  [super mouseDown:event];
-}
-
-- (void)mouseUp:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-release", 1, location.x, location.y);
-  [super mouseUp:event];
-}
-
-- (void)rightMouseDown:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-press", 2, location.x, location.y);
-  [super rightMouseDown:event];
-}
-
-- (void)rightMouseUp:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-release", 2, location.x, location.y);
-  [super rightMouseUp:event];
-}
-
-- (void)otherMouseDown:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-press", [event buttonNumber], location.x, location.y);
-  [super otherMouseDown:event];
-}
-
-- (void)otherMouseUp:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-button-release", [event buttonNumber], location.x, location.y);
-  [super otherMouseUp:event];
-}
-
-- (void)mouseMoved:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-move", event.buttonNumber, location.x, location.y);
-  [super mouseMoved:event];
-}
-
-- (void)mouseDragged:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-move", event.buttonNumber, location.x, location.y);
-  [super mouseDragged:event];
-}
-
-- (void)rightMouseDragged:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-move", event.buttonNumber, location.x, location.y);
-  [super rightMouseDragged:event];
-}
-
-- (void)otherMouseDragged:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_mouse_event (GST_GL_WINDOW (window_cocoa),
-      "mouse-move", event.buttonNumber, location.x, location.y);
-  [super otherMouseDragged:event];
-}
-
-- (void)scrollWheel:(NSEvent *)event {
-  NSPoint location = [self convertPoint:[event locationInWindow] fromView:nil];
-  gst_gl_window_send_scroll_event (GST_GL_WINDOW (window_cocoa),
-      location.x, location.y, [event deltaX], [event deltaY]);
-  [super scrollWheel:event];
-}
-
 - (void) dealloc {
   self->layer = nil;
 }
@@ -932,14 +735,10 @@ close_window_cb (gpointer data)
     return NO;
 }
 
-- (void)viewDidChangeBackingProperties {
-    self.layer.contentsScale = [[self window] backingScaleFactor];
-}
-
 @end
 
 void
-_gst_gl_invoke_on_main (GstGLWindowCB func, gpointer data, GDestroyNotify notify)
+_invoke_on_main (GstGLWindowCB func, gpointer data, GDestroyNotify notify)
 {
   if ([NSThread isMainThread]) {
     func (data);
